@@ -1,7 +1,7 @@
 use Apache ();
 use strict;
 
-my $tests = 24;
+my $tests = 29;
 my $i;
 my $r = Apache->request;
 $r->content_type("text/plain");
@@ -41,11 +41,19 @@ my(%err_headers_out) = $r->headers_out;
 test ++$i, keys %err_headers_out;
 #test ++$i, $r->err_header_out("Content-Type");
 $r->err_header_out('X-Die' => "uhoh"); 
-#$r->err_headers_out('X-Die' => "uhoh"); #test compatible
 test ++$i, $r->err_header_out("X-Die") eq "uhoh";
 
+$r->notes("FOO", 1); 
+$r->notes("ANoteKey", "TRUE");
+test ++$i, $r->notes("ANoteKey");
 test ++$i, $r->content_type;
 test ++$i, $r->handler;
+
+$r->header_out(ByeBye => "TRUE");
+test ++$i, $r->header_out("ByeBye");
+$r->header_out(ByeBye => undef);
+test ++$i, not $r->header_out("ByeBye");
+
 #content_encoding
 #content_language
 #no_cache
@@ -54,6 +62,10 @@ test ++$i, $r->handler;
 #test ++$i, $r->path_info;
 #test ++$i, $r->query_string;
 
+#just make sure we can actually call these
+test ++$i, $r->satisfies || 1;
+test ++$i, $r->some_auth_required || 1;
+
 #dir_config
 
 my $c = $r->connection;
